/*
 * Copyright SpringSource Inc 2007,2009.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.springsource.util.math;

/**
 * This class is thread safe.
 * 
 * @author Glyn Normington
 * @param <T> type of Comparable element
 * @since 1.0
 */

public final class Range<T extends Comparable<? super T>> {

	public static final String FLOOR_INCLUSIVE_DELIMITER = "[";

	public static final String CEILING_INCLUSIVE_DELIMITER = "]";

	public static final String FLOOR_EXCLUSIVE_DELIMITER = "(";

	public static final String CEILING_EXCLUSIVE_DELIMITER = ")";

	public static final String SEPARATOR = ",";

	private final T floor, ceiling;

	private final boolean floorInc, ceilingInc;

	public Range(T floor, boolean floorInc, T ceiling, boolean ceilingInc) {
		this.floor = floor;
		this.floorInc = floorInc;
		this.ceiling = ceiling;
		this.ceilingInc = ceilingInc;
	}

	public boolean contains(T t) {
		int minCheck = this.floorInc ? 0 : 1;
		int maxCheck = this.ceilingInc ? 0 : -1;
		return t.compareTo(this.floor) >= minCheck
				&& t.compareTo(this.ceiling) <= maxCheck;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuffer result = new StringBuffer();
		result.append(this.floorInc ? FLOOR_INCLUSIVE_DELIMITER
				: FLOOR_EXCLUSIVE_DELIMITER);
		result.append(this.floor.toString());
		result.append(SEPARATOR);
		result.append(this.ceiling.toString());
		result.append(this.ceilingInc ? CEILING_INCLUSIVE_DELIMITER
				: CEILING_EXCLUSIVE_DELIMITER);
		return result.toString();
	}
}
